/*
greatly inspired by MisifitMaid's Static Loading Screen plugin
https://openplanet.dev/plugin/static-loading-screen
*/

namespace CustomLoadingScreen {
    UI::Texture@ currentScreen;
    string currentScreenPath;
    string[] candidates;
    bool fired = false;

    void init() {
        Meta::Plugin@ bls = Meta::GetPluginFromID("BetterLoadingScreen");
        Meta::Plugin@ sls = Meta::GetPluginFromID("static-loading-screen");
        if ((bls !is null && bls.Enabled) || (sls !is null && sls.Enabled)) { // stops custom loading screen if betterloadingscreen or staticloadingscreen is on
            UI::ShowNotification(
                Meta::ExecutingPlugin().Name,
                "BetterLoadingScreen or static loading screen is enabled, this plugin wont make a loading screen!"
            );
            while ((bls !is null && bls.Enabled) || (sls !is null && sls.Enabled)) {
                yield();
            }
        }

        imageWatcherOnce();
        startnew(imageWatcher);
        startnew(populateLoadScreen);
        startnew(previewRotator);
    }

    void populateLoadScreen() {
        if (candidates.Length == 0) {
            IO::FileSource iofs("default.png");
            @currentScreen = UI::LoadTexture(iofs.Read(iofs.Size()));
            currentScreenPath = "";
            trace("[PP] loading placeholder image");
        }
        else {
            uint pick = Math::Rand(0, candidates.Length);
            if (candidates[pick] == currentScreenPath) {
                trace("[PP] Keeping existing image loaded: " + candidates[pick]);
                return;
            }
            if (candidates[pick] == currentScreenPath) {
                trace("[SLS] Keeping existing image loaded: " + candidates[pick]);    
                return;
            }
            IO::File iof(candidates[pick], IO::FileMode::Read);
            @currentScreen = UI::LoadTexture(iof.Read(iof.Size()));
            currentScreenPath = candidates[pick];
            iof.Close();
            trace("[PP] loading image: " + candidates[pick]);
        }
    }

    void previewRotator() {
        while(true) {
            sleep(5000);
            if (bgPreview) populateLoadScreen();
        }
    }

    void imageWatcher() {
        while(true) {
            sleep(30000);
            imageWatcherOnce();
        }
    }

    void imageWatcherOnce() {
        candidates = getImageList();
    }

    void downloadOneImage(const string &in url, string folder, string fileName) {
        Net::HttpRequest@ req = Net::HttpGet(url);
        while(!req.Finished()) yield();
        if (req.ResponseCode() != 200) {
            error("[PP] bad response code. Check network log!");
            trace(url);
            return;
        }

        trace("[PP] saving " + fileName + ".png");
        IO::File file(folder+"/"+fileName + ".png", IO::FileMode::Write);
        file.Write(req.Buffer());
        file.Close();
    }

    void downloadImages (string folder) {
        Net::HttpRequest@ req = Net::HttpGet("https://openplanet.dev/plugin/pp/config/loading_screens");
        while (!req.Finished()) yield();

        Json::Value json = Json::Parse(req.String());

        if (json.GetType() != Json::Type::Array) {
            return;
        }

        for (uint i = 0; i < json.Length; i++) {
            if (json[i].GetType() != Json::Type::Array) continue;
            if (json[i].Length == 0 || json[i][0].GetType() != Json::Type::String) continue;
            IO::CreateFolder(folder+"/"+string(json[i][0]));
            for (uint j = 1; j < json[i].Length; j++) {
                if (json[i][j].GetType() != Json::Type::String) continue;
                downloadOneImage(string(json[i][j]), folder+"/"+string(json[i][0]), string(json[i][0]) + "-" + j);
            }
        }
    }

    string[] getMapImages() {
        string mapsFolder = imageFolder + "/Maps";
        if (!IO::FolderExists(mapsFolder)) {
            IO::CreateFolder(mapsFolder);
            downloadImages(mapsFolder);
        }
        string[] images;
        dictionary maps = {
            {"CHOOSE-YOUR-CAR", mapChooseYourCar}, 
            {"ICY-PIGS", mapIcyPigs}, 
            {"no-clue", mapNoClue}, 
            {"one-section-at-the-time", mapOneSectionAtTheTime},
            {"ONO", mapOno},
            {"snow-on-pipes", mapSnowOnPipes}
        };
        string[] keys = maps.GetKeys();
        for (uint i = 0; i < keys.Length; i++) {
            if (!bool(maps[keys[i]])) continue;
            string[] x = IO::IndexFolder(mapsFolder+keys[i], true);
            for (uint j = 0; j < x.Length; j++) {
                images.InsertLast(x[j]);
            }
        }
        return images;
    }

    string[] getImageList() {
        if (imageFolder.EndsWith("/")) {
            imageFolder = imageFolder.SubStr(0, imageFolder.Length-1);
        }
        if (!IO::FolderExists(imageFolder)) { // create imagefolder if none exist
            IO::CreateFolder(imageFolder);
        }

        string[] filtered; // only takes files witch can be displayed
        string[] supportedFormats = getSupportedFormats(); // fetches all image formats witch can be displayed
        
        filtered = getMapImages();

        if (useCustomLoadingScreens) {
            string[] files = IO::IndexFolder(imageFolder, true); // fetches all files in folder and subfolders
            for (uint i = 0; i < files.Length; i++) { // checks if files are of the supported format
                if (i % 500 == 0) yield();
                bool supported = false;
                for (uint j = 0; j < supportedFormats.Length; j++) {
                    if (files[i].ToLower().EndsWith(supportedFormats[j])) {
                        supported = true;
                        break;
                    }
                }
                if (supported) {
                    filtered.InsertLast(files[i]);
                }
            }
        }
        
        return filtered;
    }

    string[] getSupportedFormats() {
        string[] supportedFormats = {
            ".jpg",
            ".jpeg",
            ".png",
            ".tga",
            ".bmp",
            ".psd",
            ".gif",
            ".hdr",
            ".pic"
        };
        return supportedFormats;
    }

    vec2 getImageSize(vec2 screen, vec2 img) {
        float r_width = screen.x / img.x;
        float r_height = screen.y / img.y;

        return img * Math::Max(r_width, r_height);
    }

    void render() {
        if (currentScreen is null) return;

        if (!bgPreview) {
            NGameLoadProgress_SMgr@ loadProgress = GetApp().LoadProgress;
            if (loadProgress is null || loadProgress.State == NGameLoadProgress::EState::Disabled) {
                if (fired) {
                    startnew(populateLoadScreen);
                }
                fired = false;
                return;
            }
        }

        UI::DrawList@ dl = UI::GetBackgroundDrawList();
        vec2 screen(Draw::GetWidth(), Draw::GetHeight());
        vec2 img = getImageSize(screen, currentScreen.GetSize());
        dl.AddImage(currentScreen, screen/2.f -  img/2.f, img);

        fired = true;
    }
}